searchData={"items":[{"type":"module","title":"NimblePublisher","doc":"`NimblePublisher` is a minimal filesystem-based publishing engine with Markdown support and\ncode highlighting.\n\n```elixir\nuse NimblePublisher,\n  build: Article,\n  from: Application.app_dir(:app_name, \"priv/articles/**/*.md\"),\n  as: :articles,\n  highlighters: [:makeup_elixir, :makeup_erlang]\n```\n\nThe example above will get all articles in the given directory,\ncall `Article.build/3` for each article, passing the filename,\nthe metadata, and the article body, and define a module attribute\nnamed `@articles` with all built articles returned by the\n`Article.build/3` function.\n\nEach article in the articles directory must have the format:\n\n    %{\n      title: \"Hello world\"\n    }\n    ---\n    Body of the \"Hello world\" article.\n\n    This is a *markdown* document with support for code highlighters:\n\n    ```elixir\n    IO.puts \"hello world\"\n    ```","ref":"NimblePublisher.html"},{"type":"module","title":"Options - NimblePublisher","doc":"* `:build` - the name of the module that will build each entry\n\n  * `:from` - a wildcard pattern where to find all entries. Files with the\n    `.md` or `.markdown` extension will be converted to Markdown with\n    `Earmark`. Other files will be kept as is.\n\n  * `:as` - the name of the module attribute to store all built entries\n\n  * `:highlighters` - which code highlighters to use. `NimblePublisher`\n    uses `Makeup` for syntax highlighting and you will need to add its\n    `.css` classes. You can generate the CSS classes by calling\n    `Makeup.stylesheet(:vim_style, \"makeup\")` inside `iex -S mix`.\n    You can replace `:vim_style` by any style of your choice\n    [defined here](https://elixir-makeup.github.io/makeup_demo/elixir.html).\n\n  * `:earmark_options` - an [`%Earmark.Options{}`](https://hexdocs.pm/earmark/Earmark.Options.html) struct\n\n  * `:parser` - custom module with a `parse/2` function that receives the file path\n    and content as params. See [Custom parser](#module-custom-parser) for more details.\n\n  * `:html_converter` - custom module with a `convert/4` function that receives the\n    extension, body, and attributes of the markdown file, as well as all options\n    as params. See [Custom HTML converter](#module-custom-html-converter) for more details.","ref":"NimblePublisher.html#module-options"},{"type":"module","title":"Examples - NimblePublisher","doc":"Let's see a complete example. First add `nimble_publisher` with\nthe desired highlighters as a dependency:\n\n    def deps do\n      [\n        {:nimble_publisher, \"~> 1.0\"},\n        {:makeup_elixir, \">= 0.0.0\"},\n        {:makeup_erlang, \">= 0.0.0\"}\n      ]\n    end\n\nIn this example, we are building a blog. Each post stays in the\n\"posts\" directory with the format:\n\n    /posts/YEAR/MONTH-DAY-ID.md\n\nA typical blog post will look like this:\n\n    # /posts/2020/04-17-hello-world.md\n    %{\n      title: \"Hello world!\",\n      author: \"José Valim\",\n      tags: ~w(hello),\n      description: \"Let's learn how to say hello world\"\n    }\n    ---\n    This is the post.\n\nTherefore, we will define a Post struct that expects all of the fields\nabove. We will also have a `:date` field that we will build from the\nfilename. Overall, it will look like this:\n\n```elixir\ndefmodule MyApp.Blog.Post do\n  @enforce_keys [:id, :author, :title, :body, :description, :tags, :date]\n  defstruct [:id, :author, :title, :body, :description, :tags, :date]\n\n  def build(filename, attrs, body) do\n    [year, month_day_id] = filename |> Path.rootname() |> Path.split() |> Enum.take(-2)\n    [month, day, id] = String.split(month_day_id, \"-\", parts: 3)\n    date = Date.from_iso8601!(\"#{year}-#{month}-#{day}\")\n    struct!(__MODULE__, [id: id, date: date, body: body] ++ Map.to_list(attrs))\n  end\nend\n```\n\nNow, we are ready to define our `MyApp.Blog` with `NimblePublisher`:\n\n```elixir\ndefmodule MyApp.Blog do\n  alias MyApp.Blog.Post\n\n  use NimblePublisher,\n    build: Post,\n    from: Application.app_dir(:my_app, \"priv/posts/**/*.md\"),\n    as: :posts,\n    highlighters: [:makeup_elixir, :makeup_erlang]\n\n  # The @posts variable is first defined by NimblePublisher.\n  # Let's further modify it by sorting all posts by descending date.\n  @posts Enum.sort_by(@posts, & &1.date, {:desc, Date})\n\n  # Let's also get all tags\n  @tags @posts |> Enum.flat_map(& &1.tags) |> Enum.uniq() |> Enum.sort()\n\n  # And finally export them\n  def all_posts, do: @posts\n  def all_tags, do: @tags\nend\n```\n\n**Important**: Avoid injecting the `@posts` attribute into multiple functions,\nas each call will make a complete copy of all posts. For example, if you want\nto define `recent_posts()` as well as `all_posts()`, DO NOT do this:\n\n```elixir\ndef all_posts, do: @posts\ndef recent_posts, do: Enum.take(@posts, 3)\n```\n\nInstead do this:\n\n```elixir\ndef all_posts, do: @posts\ndef recent_posts, do: Enum.take(all_posts(), 3)\n```","ref":"NimblePublisher.html#module-examples"},{"type":"module","title":"Other helpers - NimblePublisher","doc":"You may want to define other helpers to traverse your published resources.\nFor example, if you want to get posts by ID or with a given tag, you can\ndefine additional functions as shown below:\n\n```elixir\ndefmodule NotFoundError do\n  defexception [:message, plug_status: 404]\nend\n\ndef get_post_by_id!(id) do\n  Enum.find(all_posts(), &(&1.id == id)) ||\n    raise NotFoundError, \"post with id=#{id} not found\"\nend\n\ndef get_posts_by_tag!(tag) do\n  case Enum.filter(all_posts(), &(tag in &1.tags)) do\n    [] -> raise NotFoundError, \"posts with tag=#{tag} not found\"\n    posts -> posts\n  end\nend\n```","ref":"NimblePublisher.html#module-other-helpers"},{"type":"module","title":"Custom parser - NimblePublisher","doc":"You may want to define a custom function to parse the content of your files.\n\n```elixir\n  use NimblePublisher,\n    ...\n    parser: Parser,\n\ndefmodule Parser do\n  def parse(path, contents) do\n    [attrs, body] = :binary.split(contents, [\"\\n---\\n\"])\n    {Jason.decode!(attrs), body}\n  end\nend\n```\n\nThe `parse/2` function from this module receives the file path and content as params.\nIt must return:\n\n  * a 2 element tuple with attributes and body - `{attrs, body}`\n  * a list of 2 element tuple with attributes and body - `[{attrs, body} | _]`","ref":"NimblePublisher.html#module-custom-parser"},{"type":"module","title":"Custom HTML converter - NimblePublisher","doc":"You can also define a custom HTML converter that will be used to convert the\nfile body (typically Markdown) into HTML. For example, you may wish to use an\nalternative Markdown parser such as [md](https://github.com/am-kantox/md).\nIf you want to use the built-in highlighting, you need to call it manually.\n\n```elixir\n  use NimblePublisher,\n    ...\n    html_converter: MarkdownConverter,\n    highlighters: [:makeup_elixir]\n```\n\n```elixir\ndefmodule MarkdownConverter do\n  def convert(filepath, body, _attrs, opts) do\n    if Path.extname(filepath) in [\".md\", \".markdown\"] do\n      highlighters = Keyword.get(opts, :highlighters, [])\n      body |> Md.generate() |> NimblePublisher.highlight(highlighters)\n    end\n  end\nend\n```\n\nThe `convert/4` function from this module receives an extension name, a body,\nthe parsed attributes from the file, and the options passed to\n`NimblePublisher`. It must return the converted body as a string.","ref":"NimblePublisher.html#module-custom-html-converter"},{"type":"module","title":"Live reloading - NimblePublisher","doc":"If you are using Phoenix, you can enable live reloading by simply telling Phoenix to watch the “posts” directory. Open up \"config/dev.exs\", search for `live_reload:` and add this to the list of patterns:\n\n```elixir\nlive_reload: [\n  patterns: [\n    ...,\n    ~r\"posts/*/.*(md)$\"\n  ]\n]\n```","ref":"NimblePublisher.html#module-live-reloading"},{"type":"module","title":"Learn more - NimblePublisher","doc":"* [Dashbit's blog post which was the foundation for NimblePublisher](https://dashbit.co/blog/welcome-to-our-blog-how-it-was-made)\n  * [Elixir School's lesson on using NimblePublisher, complete with Phoenix integration](https://elixirschool.com/en/lessons/libraries/nimble-publisher/)","ref":"NimblePublisher.html#module-learn-more"},{"type":"function","title":"NimblePublisher.highlight/2","doc":"Highlights all code blocks in an already generated HTML document.\n\nIt uses Makeup and expects the existing highlighters applications to\nbe already started.\n\nOptions:\n\n  * `:regex` - the regex used to find code blocks in the HTML document. The regex\n    should have two capture groups: the first one should be the language name\n    and the second should contain the code to be highlighted. The default\n    regex to match with generated HTML documents is:\n\n        ~r/  ([^<]*)  /","ref":"NimblePublisher.html#highlight/2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}